"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFileReferences = getFileReferences;
function getFileReferences(fileName) {
    const { typescript: ts, languageService } = this;
    const program = languageService.getProgram();
    const result = languageService.getFileReferences(fileName);
    const references = new Set(result);
    const skippedReferences = [];
    for (const reference of result) {
        if (reference.fileName.endsWith('.vue')) {
            continue;
        }
        const sourceFile = program.getSourceFile(reference.fileName);
        if (!sourceFile) {
            continue;
        }
        visit(sourceFile, reference, sourceFile);
    }
    for (const reference of skippedReferences) {
        references.delete(reference);
    }
    return [...references].map(({ fileName, textSpan }) => {
        const sourceFile = program.getSourceFile(fileName);
        const start = sourceFile.getLineAndCharacterOfPosition(textSpan.start);
        const end = sourceFile.getLineAndCharacterOfPosition(textSpan.start + textSpan.length);
        return {
            fileName,
            range: {
                start,
                end,
            },
        };
    });
    function visit(node, reference, sourceFile) {
        if (ts.isPropertySignature(node) && node.type) {
            proxy(node.name, node.type, reference, sourceFile);
        }
        else if (ts.isVariableDeclaration(node) && ts.isIdentifier(node.name) && node.type && !node.initializer) {
            proxy(node.name, node.type, reference, sourceFile, true);
        }
        else {
            ts.forEachChild(node, child => visit(child, reference, sourceFile));
        }
    }
    function proxy(name, type, reference, sourceFile, skip = false) {
        const { textSpan, fileName } = reference;
        const start = type.getStart(sourceFile);
        const end = type.getEnd();
        if (start > textSpan.start || end < textSpan.start + textSpan.length) {
            return;
        }
        if (skip) {
            skippedReferences.push(reference);
            return;
        }
        const pos = name.getStart(sourceFile);
        const res = languageService.findReferences(fileName, pos) ?? [];
        const items = res.flatMap(({ references }) => references);
        if (items.length) {
            for (const reference of items) {
                if (reference.isDefinition) {
                    continue;
                }
                references.add(reference);
            }
            skippedReferences.push(reference);
        }
    }
}
//# sourceMappingURL=getFileReferences.js.map